"""Download ERA-5 data from the Copernicus Climate Data Store."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import os
from joblib import Parallel, delayed

# externals
import cdsapi
import numpy as np

# locals
from climax.core.constants import ERA5_P_VARIABLES, ERA5_VARIABLES
from climax.main.io import ERA5_PATH

# pressure levels
pressure_levels = ['850', '500']

# time period
years = [str(y) for y in np.arange(1981, 2011)]
month = [str(m) for m in np.arange(1, 13)]
days = [str(d) for d in np.arange(1, 32)]
time = ["{:02d}:00".format(t) for t in np.arange(0, 24)]

# area of interest (Alps): North, West, South, East
area = [52, 2, 40, 20]

# ERA5 download configuration dictionary
CONFIG = {
    'product_type': 'reanalysis',
    'month': month,
    'day': days,
    'time': time,
    'format': 'netcdf',
    'area': area
}


if __name__ == '__main__':

    # initialize client
    c = cdsapi.Client()

    # download data for the different variables
    for var in ERA5_VARIABLES:
        # create output directory
        output = ERA5_PATH.joinpath('Downloads', var)
        if not output.exists():
            output.mkdir(parents=True, exist_ok=True)

        # create output files
        files = [output.joinpath('_'.join(['ERA5', var, year]) + '.nc') for
                 year in years]

        # check whether to download variable on pressure levels or single level
        if var in ERA5_P_VARIABLES:
            # download configuration: ERA5 variable on pressure levels
            product = 'reanalysis-era5-pressure-levels'
            CONFIG['pressure_level'] = pressure_levels  # pressure levels
        else:
            # download configuration: ERA5 variable on single level
            product = 'reanalysis-era5-single-levels'
            if 'pressure_level' in CONFIG:
                _ = CONFIG.pop('pressure_level') # remove pressure levels

        # "orography" variable was recently renamed to "geopotential": this
        # results in conflicting file names
        var = 'geopotential' if var == 'orography' else var

        # split the download to the different years: CDS API cannot handle
        # requests over the whole time period
        Parallel(n_jobs=min(len(years), os.cpu_count()), verbose=51)(
            delayed(c.retrieve)(
                product, {**CONFIG, **{'variable': var, 'year': year}}, file)
            for file, year in zip(files, years) if not file.exists())
